# Generated by Django 3.2.5 on 2023-01-25 08:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bank_sync', '0013_alter_ipndata_ipn'),
    ]

    operations = [
        migrations.RenameField(
            model_name='ipn',
            old_name='client_secret',
            new_name='signature',
        ),
        migrations.AddField(
            model_name='ipn',
            name='country_code',
            field=models.CharField(default='KE', max_length=5),
        ),
        migrations.AddField(
            model_name='ipn',
            name='currency_code',
            field=models.CharField(default='KES', max_length=5),
        ),
    ]
